@@ -1,16 +1,12 @@
-"""
-    nemreader.nem_reader
-    ~~~~~
-    Read MDFF format
-"""
-
 import csv
 import logging
 import os
 import zipfile
 from datetime import datetime, timedelta
 from itertools import chain, islice
-from typing import Any, Dict, Iterable, List, Optional
+from typing import Any, Dict, Generator, Iterable, List, Optional, Tuple
+
+import pandas as pd
 
 from .nem_objects import (
     B2BDetails12,
@@ -19,20 +15,173 @@ from .nem_objects import (
     EventRecord,
     HeaderRecord,
     IntervalRecord,
-    NEMFile,
+    NEMData,
+    NEMReadings,
     NmiDetails,
     Reading,
 )
+from .split_days import make_set_interval, split_multiday_reads
 
 log = logging.getLogger(__name__)
 
 
+class NEMFile:
+    """An NEM file object"""
+
+    def __init__(self, file_path: str, strict: bool = False) -> None:
+
+        self.file_path = file_path
+        self.strict = strict
+        self.nmis: set = set()
+        self.nmi_channels: dict = {}
+
+    def __repr__(self):
+        return f"<NEMFile {self.file_path}>"
+
+    @property
+    def zipped(self) -> bool:
+        """Check whether file is zipped or not"""
+        _, file_extension = os.path.splitext(self.file_path)
+        if file_extension.lower() == ".zip":
+            return True
+        return False
+
+    def parse_nem_file(self, nem_file, file_name="") -> NEMReadings:
+        """Parse NEM file and return meter readings named tuple"""
+        reader = csv.reader(nem_file, delimiter=",")
+        first_row = next(reader, None)
+
+        # Some Powercor/Citipower files have empty line at start, skip if so.
+        if not first_row:
+            first_row = next(reader, None)
+
+        try:
+            record_indicator = int(first_row[0])
+        except Exception:
+            record_indicator = 0
+
+        if not first_row or record_indicator != 100:
+            if self.strict:
+                raise ValueError("NEM Files must start with a 100 row")
+            else:
+                log.warning("Missing header (100) row, assuming NEM12.")
+                header = HeaderRecord("NEM12", None, "", "", file_name, assumed=True)
+        else:
+            header = parse_100_row(first_row, file_name)
+            if header.version_header not in ["NEM12", "NEM13"]:
+                raise ValueError("Invalid NEM version {}".format(header.version_header))
+
+        self.header = header
+        if header.assumed:
+            # We have to parse the first row again so we don't miss any data.
+            reader = chain([first_row], reader)
+            return parse_nem12_rows(reader, file_name=nem_file)
+        if header.version_header == "NEM12":
+            return parse_nem12_rows(reader, file_name=nem_file)
+        else:
+            return parse_nem13_rows(reader)
+
+    def nem_data(self) -> NEMData:
+        """Return data in legacy data format"""
+        if self.zipped:
+            log.debug("Extracting zip file")
+            with zipfile.ZipFile(self.file_path, "r") as archive:
+                for csv_file in archive.namelist():
+                    with archive.open(csv_file) as csv_text:
+                        # Zip file is open in binary mode
+                        # So decode then convert back to list
+                        nmi_file = csv_text.read().decode("utf-8").splitlines()
+                        reads = self.parse_nem_file(nmi_file, file_name=csv_file)
+                        for nmi in reads.transactions.keys():
+                            self.nmis.add(nmi)
+                            suffixes = list(reads.transactions[nmi].keys())
+                            self.nmi_channels[nmi] = suffixes
+                        return NEMData(
+                            header=self.header,
+                            readings=reads.readings,
+                            transactions=reads.transactions,
+                        )
+
+        with open(self.file_path) as nmi_file:
+            reads = self.parse_nem_file(nmi_file)
+            for nmi in reads.transactions.keys():
+                self.nmis.add(nmi)
+                suffixes = list(reads.transactions[nmi].keys())
+                self.nmi_channels[nmi] = suffixes
+            return NEMData(
+                header=self.header,
+                readings=reads.readings,
+                transactions=reads.transactions,
+            )
+
+    def get_data_frame(
+        self, split_days: bool = False, set_interval: Optional[int] = None
+    ) -> pd.DataFrame:
+        """Return NEMData as a DataFrame"""
+        nd = self.nem_data()
+        frames = []
+        for nmi in nd.readings.keys():
+            for suffix in nd.readings[nmi].keys():
+                reads = nd.readings[nmi][suffix]
+
+                if split_days or set_interval:
+                    reads = list(split_multiday_reads(reads))
+
+                if set_interval:
+                    reads = list(make_set_interval(reads, set_interval))
+
+                data = {
+                    "nmi": [nmi for _ in range(len(reads))],
+                    "suffix": [suffix for _ in range(len(reads))],
+                    "serno": [x.meter_serial_number for x in reads],
+                    "t_start": [x.t_start for x in reads],
+                    "t_end": [x.t_end for x in reads],
+                    "value": [x.read_value for x in reads],
+                    "quality": [x.quality_method for x in reads],
+                    "evt_code": [x.event_code for x in reads],
+                    "evt_desc": [x.event_desc for x in reads],
+                }
+                frames.append(pd.DataFrame(data))
+        return pd.concat(frames)
+
+    def get_pivot_data_frame(
+        self,
+        split_days: bool = False,
+        set_interval: Optional[int] = None,
+        include_serno: bool = False,
+    ) -> pd.DataFrame:
+        """Return NEMData as a DataFrame with suffix columns"""
+        df = self.get_data_frame(split_days, set_interval)
+        pivot_index = ["nmi", "t_start", "t_end", "quality", "evt_code", "evt_desc"]
+        if include_serno:
+            pivot_index.append("serno")
+        df_pivoted = df.pivot(
+            index=pivot_index,
+            columns="suffix",
+            values="value",
+        ).reset_index()
+        return df_pivoted
+
+    def get_per_nmi_dfs(
+        self,
+        split_days: bool = False,
+        set_interval: Optional[int] = None,
+        include_serno: bool = False,
+    ) -> Generator[Tuple[str, pd.DataFrame], None, None]:
+        df = self.get_pivot_data_frame(split_days, set_interval, include_serno)
+        nmis = df.nmi.unique()
+        for nmi in nmis:
+            nmi_df = df[(df["nmi"] == nmi)]
+            del nmi_df["nmi"]
+            yield nmi, nmi_df
+
+
 def flatten_list(l: List[list]) -> list:
     """takes a list of lists, l and returns a flat list"""
     return [v for inner_l in l for v in inner_l]
 
 
-def read_nem_file(file_path: str, ignore_missing_header=False) -> NEMFile:
+def read_nem_file(file_path: str, ignore_missing_header=False) -> NEMData:
     """Read in NEM file and return meter readings named tuple
 
     :param file_path: The NEM file to process
@@ -41,70 +190,8 @@ def read_nem_file(file_path: str, ignore_missing_header=False) -> NEMFile:
     :returns: The file that was created
     """
 
-    _, file_extension = os.path.splitext(file_path)
-    if file_extension.lower() == ".zip":
-        log.debug("Extracting zip file")
-        with zipfile.ZipFile(file_path, "r") as archive:
-            for csv_file in archive.namelist():
-                with archive.open(csv_file) as csv_text:
-                    # Zip file is open in binary mode
-                    # So decode then convert back to list
-                    nmi_file = csv_text.read().decode("utf-8").splitlines()
-
-                    return parse_nem_file(
-                        nmi_file,
-                        file_name=csv_file,
-                        ignore_missing_header=ignore_missing_header,
-                    )
-
-    with open(file_path) as nmi_file:
-        return parse_nem_file(nmi_file, ignore_missing_header=ignore_missing_header)
-
-
-def parse_nem_file(nem_file, file_name="", ignore_missing_header=False) -> NEMFile:
-    """Parse NEM file and return meter readings named tuple"""
-    reader = csv.reader(nem_file, delimiter=",")
-    first_row = next(reader, None)
-
-    # Some Powercor/Citipower files have empty line at start, skip if so.
-    if not first_row:
-        first_row = next(reader, None)
-
-    header = parse_header_row(
-        first_row,
-        ignore_missing_header=ignore_missing_header,
-        file_name=getattr(nem_file, "name", file_name),
-    )
-
-    if header.assumed:
-        # We have to parse the first row again so we don't miss any data.
-        reader = chain([first_row], reader)
-        return parse_nem12_rows(reader, header=header, file_name=nem_file)
-    if header.version_header == "NEM12":
-        return parse_nem12_rows(reader, header=header, file_name=nem_file)
-    else:
-        return parse_nem13_rows(reader, header=header, file_name=nem_file)
-
-
-def parse_header_row(
-    row: List[Any], ignore_missing_header=False, file_name=None
-) -> HeaderRecord:
-    """Parse first row of NEM file"""
-
-    record_indicator = int(row[0])
-    if record_indicator != 100:
-        if ignore_missing_header:
-            log.warning("Missing header (100) row, assuming NEM12.")
-            header = HeaderRecord("NEM12", None, "", "", file_name, True)
-        else:
-            raise ValueError("NEM Files must start with a 100 row")
-    else:
-        header = parse_100_row(row, file_name)
-        if header.version_header not in ["NEM12", "NEM13"]:
-            raise ValueError("Invalid NEM version {}".format(header.version_header))
-
-    log.debug("Parsing %s file %s ...", header.version_header, file_name)
-    return header
+    nf = NEMFile(file_path, strict=ignore_missing_header)
+    return nf.nem_data()
 
 
 def parse_100_row(row: List[Any], file_name: str) -> HeaderRecord:
@@ -114,9 +201,7 @@ def parse_100_row(row: List[Any], file_name: str) -> HeaderRecord:
     )
 
 
-def parse_nem12_rows(
-    nem_list: Iterable, header: HeaderRecord, file_name=None
-) -> NEMFile:
+def parse_nem12_rows(nem_list: Iterable, file_name=None) -> NEMReadings:
     """Parse NEM row iterator and return meter readings named tuple"""
     # readings nested by NMI then channel
     readings: Dict[str, Dict[str, List[Reading]]] = {}
@@ -201,12 +286,10 @@ def parse_nem12_rows(
     if not observed_900_record:
         log.warning("Missing end of data (900) row.")
 
-    return NEMFile(header=header, readings=readings, transactions=trans)
+    return NEMReadings(readings=readings, transactions=trans)
 
 
-def parse_nem13_rows(
-    nem_list: Iterable, header: HeaderRecord, file_name=None
-) -> NEMFile:
+def parse_nem13_rows(nem_list: Iterable) -> NEMReadings:
     """Parse NEM row iterator and return meter readings named tuple"""
     # readings nested by NMI then channel
     readings: Dict[str, Dict[str, List[Reading]]] = {}
@@ -248,7 +331,7 @@ def parse_nem13_rows(
             log.warning(
                 "Record indicator %s not supported and was skipped", record_indicator
             )
-    return NEMFile(header=header, readings=readings, transactions=trans)
+    return NEMReadings(readings=readings, transactions=trans)
 
 
 def calculate_manual_reading(basic_data: BasicMeterData) -> Reading:
